@@ -2,7 +2,7 @@
 # Modified by Cameron Dale
 # see LICENSE.txt for license information
 #
-# $Id: HTTPDownloader.py 400 2008-07-19 23:52:10Z camrdale-guest $
+# $Id: HTTPDownloader.py 425 2010-03-21 04:50:44Z camrdale $
 
 """Manage downloading pieces over HTTP.
 
@@ -10,12 +10,11 @@
 @var logger: the logger to send all log messages to for this module
 @type VERSION: C{string}
 @var VERSION: the UserAgent identifier sent to all sites
-@type haveall: L{haveComplete}
-@var haveall: instance of the seed's bitfield
 
 """
 
 from DebTorrent.CurrentRateMeasure import Measure
+from DebTorrent.bitfield import Bitfield
 from random import randint
 from urlparse import urlparse
 from httplib import HTTPConnection, BadStatusLine
@@ -28,21 +27,6 @@ logger = logging.getLogger('DebTorrent.BT1.HTTPDownloader')
 
 VERSION = product_name+'/'+version_short
 
-class haveComplete:
-    """Dummy class similar to L{Debtorrent.bitfield.Bitfield}.
-    
-    This class represents the HTTP seed's bitfield, which is always complete
-    and has every piece because it is a seed.
-    
-    """
-    def complete(self):
-        """Dummy function to always return true."""
-        return True
-    def __getitem__(self, x):
-        """Dummy function to always return true."""
-        return True
-haveall = haveComplete()
-
 class SingleDownload:
     """Control HTTP downloads from a single site.
     
@@ -62,7 +46,7 @@ class SingleDownload:
     @type query: C{string}
     @ivar query: the query component from the L{baseurl}
     @type headers: C{dictionary}
-    @ivar headres: the HTTP headers to send in the request
+    @ivar headers: the HTTP headers to send in the request
     @type measure: L{DebTorrent.CurrentRateMeasure.Measure}
     @ivar measure: tracks the download rate from the site
     @type index: C{int}
@@ -111,6 +95,7 @@ class SingleDownload:
         
         self.downloader = downloader
         self.baseurl = url
+        self.have = Bitfield(length = downloader.numpieces, defaultValue = True)
         try:
             (scheme, self.netloc, path, params, query, fragment) = urlparse(url)
         except:
@@ -200,11 +185,11 @@ class SingleDownload:
         if self.downloader.picker.am_I_complete():
             self.downloader.downloads.remove(self)
             return
-        self.index = self.downloader.picker.next(haveall, self._want)
+        self.index = self.downloader.picker.next(self.have, self._want)
         if ( self.index is None and not self.endflag
                      and not self.downloader.peerdownloader.has_downloaders() ):
             self.endflag = True
-            self.index = self.downloader.picker.next(haveall, self._want)
+            self.index = self.downloader.picker.next(self.have, self._want)
         if self.index is None:
             self.endflag = True
             self.resched()
@@ -300,6 +285,18 @@ class SingleDownload:
         
         """
         
+        if self.connection_status == 404:
+            logger.warning('http seed is missing piece: ' + str(self.index))
+            if self.have.complete():
+                self.have[self.index] = False
+                for i in xrange(len(self.have)):
+                    if self.have[i]:
+                        self.downloader.picker.got_have(i)
+            else:
+                if self.have[self.index]:
+                    self.downloader.picker.lost_have(i)
+                self.have[self.index] = False
+            return False
         if self.connection_status not in [200, 206]:
             logger.warning('bad status from http seed: '+str(self.connection_status))
             self.errorcount += 1
@@ -429,7 +426,7 @@ class HTTPDownloader:
     
     def __init__(self, storage, picker, rawserver,
                  finflag, errorfunc, peerdownloader,
-                 max_rate_period, infohash, measurefunc, gotpiecefunc,
+                 max_rate_period, numpieces, measurefunc, gotpiecefunc,
                  filenamefunc):
         """Initialize the instance.
         
@@ -448,8 +445,8 @@ class HTTPDownloader:
         @type max_rate_period: C{float}
         @param max_rate_period: maximum amount of time to guess the current 
             rate estimate represents
-        @type infohash: C{string}
-        @param infohash: the info hash
+        @type numpieces: C{int}
+        @param numpieces: total number of pieces in the download
         @type measurefunc: C{method}
         @param measurefunc: the method to call to add downloaded data to the total
             download rate measurement
@@ -466,7 +463,7 @@ class HTTPDownloader:
         self.finflag = finflag
         self.errorfunc = errorfunc
         self.peerdownloader = peerdownloader
-        self.infohash = infohash
+        self.numpieces = numpieces
         self.max_rate_period = max_rate_period
         self.gotpiecefunc = gotpiecefunc
         self.measurefunc = measurefunc
