@@ -2,7 +2,7 @@
 # Modified by Cameron Dale
 # see LICENSE.txt for license information
 #
-# $Id: PiecePicker.py 266 2007-08-18 02:06:35Z camrdale-guest $
+# $Id: PiecePicker.py 342 2008-01-23 03:21:02Z camrdale-guest $
 
 """Choose pieces to download.
 
@@ -15,7 +15,7 @@ from random import randrange, shuffle
 from DebTorrent.clock import clock
 import logging
 
-logger = logging.getLogger('DebTorrent.BT1.Rerequester')
+logger = logging.getLogger('DebTorrent.BT1.PiecePicker')
 
 class PiecePicker:
     """Choose pieces to download.
