@@ -2,7 +2,7 @@
 # Modified by Cameron Dale
 # see LICENSE.txt for license information
 #
-# $Id: Storage.py 266 2007-08-18 02:06:35Z camrdale-guest $
+# $Id: Storage.py 347 2008-01-24 00:43:09Z camrdale-guest $
 
 """Low-level writing of files.
 
@@ -181,10 +181,18 @@ class Storage:
             if length == 0:
                 self.file_ranges.append(None)
                 self.working_ranges.append([])
-                if total == piece_total and cur_piece > 0:
-                    self.file_pieces.append((cur_piece-1, cur_piece-1))
-                else:
-                    self.file_pieces.append((cur_piece, cur_piece))
+                
+                # Assign any zero-length pieces to the empty file
+                start_piece = cur_piece
+                for cur_piece in xrange(start_piece,len(self.piece_lengths)):
+                    if self.piece_lengths[cur_piece] > 0:
+                        break
+                    self.piece_files[cur_piece] = (0L, 0L, 0L, '')
+                end_piece = cur_piece-1
+                if self.piece_lengths[cur_piece] == 0:
+                    # Special case if the last file in a torrent is empty
+                    end_piece = cur_piece
+                self.file_pieces.append((start_piece, end_piece))
             else:
                 range = (total, total + length, 0, file)
                 self.file_ranges.append(range)
@@ -807,39 +815,25 @@ class Storage:
         
         Pickled data format::
     
-            d['files'] = [ file #, size, mtime {, file #, size, mtime...} ]
-                        file # in torrent, and the size and last modification
+            d['files'] = {file name: [size, mtime], file name: [size, mtime], ...}
+                        file name in torrent, and the size and last modification
                         time for those files.  Missing files are either empty
                         or disabled.
-            d['partial files'] = [ name, size, mtime... ]
-                        Names, sizes and last modification times of files containing
-                        partial piece data.  Filenames go by the following convention:
-                        {file #, 0-based}{nothing, "b" or "e"}
-                        eg: "0e" "3" "4b" "4e"
-                        Where "b" specifies the partial data for the first piece in
-                        the file, "e" the last piece, and no letter signifying that
-                        the file is disabled but is smaller than one piece, and that
-                        all the data is cached inside so adjacent files may be
-                        verified.
         
         @rtype: C{dictionary}
         @return: the pickled current status of the download
         
         """
         
-        files = []
-        pfiles = []
+        files = {}
         for i in xrange(len(self.files)):
             if not self.files[i][1]:    # length == 0
                 continue
             if self.disabled[i]:
-                # Removed due to files always ending on pieces
-                #for start, end, offset, file in self._get_disabled_ranges(i)[2]:
-                #    pfiles.extend([basename(file),getsize(file),int(getmtime(file))])
                 continue
             file = self.files[i][0]
-            files.extend([i,getsize(file),int(getmtime(file))])
-        return {'files': files, 'partial files': pfiles}
+            files[file] = [getsize(file), int(getmtime(file))]
+        return {'files': files}
 
 
     def unpickle(self, data):
@@ -855,19 +849,9 @@ class Storage:
         """
 
         try:
-            files = {}
-            pfiles = {}
-            l = data['files']
-            assert len(l) % 3 == 0
-            l = [l[x:x+3] for x in xrange(0,len(l),3)]
-            for f, size, mtime in l:
-                files[f] = (size, mtime)
-            l = data.get('partial files',[])
-            assert len(l) % 3 == 0
-            l = [l[x:x+3] for x in xrange(0,len(l),3)]
-            for file, size, mtime in l:
-                pfiles[file] = (size, mtime)
-
+            files = data['files']
+            logger.debug('saved file characteristics: %r', files)
+            
             valid_pieces = {}
             for i in xrange(len(self.files)):
                 if self.disabled[i]:
@@ -886,7 +870,7 @@ class Storage:
             def test(old, size, mtime):
                 """Test that the file has not changed since the status save.                
                 
-                @type old: C{tuple} of (C{long}, C{long})
+                @type old: C{list} of [C{long}, C{long}]
                 @param old: the previous size and modification time of the file
                 @type size: C{long}
                 @param size: the current size of the file
@@ -899,22 +883,24 @@ class Storage:
                 
                 oldsize, oldmtime = old
                 if size != oldsize:
+                    logger.debug('Size of file has changed: %d to %d', oldsize, size)
                     return False
                 if mtime > oldmtime+1:
+                    logger.debug('Time of file has changed: %d to %d', oldmtime, mtime)
                     return False
                 if mtime < oldmtime-1:
+                    logger.debug('Time of file has changed: %d to %d', oldmtime, mtime)
                     return False
                 return True
 
             for i in xrange(len(self.files)):
                 if self.disabled[i]:
-                    # Simplified due to file boundaries being piece boundaries
                     continue
                 file, size = self.files[i]
                 if not size:
                     continue
-                if ( not files.has_key(i)
-                     or not test(files[i],getsize(file),getmtime(file)) ):
+                if ( not files.has_key(file)
+                     or not test(files[file],getsize(file),getmtime(file)) ):
                     start, end, offset, file = self.file_ranges[i]
                     start_piece, end_piece = self.file_pieces[i]
                     logger.debug('removing '+file)
