@@ -3,7 +3,7 @@
 # Modified by Cameron Dale
 # see LICENSE.txt for license information
 #
-# $Id: makemetafile.py 383 2008-06-20 03:30:24Z camrdale-guest $
+# $Id: makemetafile.py 426 2010-03-21 05:03:59Z camrdale $
 
 """Create a torrent file or data structure.
 
@@ -33,7 +33,7 @@ import gzip
 from bz2 import BZ2File
 from StringIO import StringIO
 from re import subn
-from debian_bundle import deb822
+from debian import deb822
 import binascii, logging
 try:
     from sys import getfilesystemencoding
@@ -131,10 +131,11 @@ def uniconvert(s, e = None):
     if not e:
         e = 'ascii'
     
-    try:
-        s = unicode(s,e)
-    except UnicodeError:
-        raise UnicodeError('bad filename: '+s)
+    if type(s) != unicode:
+        try:
+            s = unicode(s,e)
+        except UnicodeError:
+            raise UnicodeError('bad filename: '+s)
     return s.encode('utf-8')
 
 def convert_all(f):
@@ -346,12 +347,12 @@ def getsubpieces(pieces_file):
         return {}
 
     for pkg in ExtraPieces.iter_paragraphs(f):
-        if len(pkg.get('SHA1', '')) > 0 and len(pkg.get('SHA1-pieces', [])) > 0:
-            sha1 = binascii.a2b_hex(pkg['SHA1'])
+        if len(str(pkg.get('SHA1', ''))) > 0 and len(pkg.get('SHA1-pieces', [])) > 0:
+            sha1 = binascii.a2b_hex(str(pkg['SHA1']))
             pieces[sha1] = ([], [])
             for piece in pkg['SHA1-pieces']:
-                pieces[sha1][0].append(binascii.a2b_hex(piece['SHA1']))
-                pieces[sha1][1].append(int(piece['size']))
+                pieces[sha1][0].append(binascii.a2b_hex(str(piece['SHA1'])))
+                pieces[sha1][1].append(int(str(piece['size'])))
 
     try:
         f.close()
@@ -391,13 +392,13 @@ def getordering(torrent_file):
 
     tor = Torrent(f)
     for header in tor:
-        if header.lower() != 'piecenumbers':
+        if str(header.lower()) != 'piecenumbers':
             # Read the headers from the file
-            headers[header] = tor[header]
+            headers[str(header)] = str(tor[header])
             
     # Read the piece ordering data from the file
     for piece in tor['PieceNumbers']:
-        pieces[int(piece['number'])] = piece['file']
+        pieces[int(str(piece['number']))] = str(piece['file'])
             
     logger.info('successfully retrieved torrent ordering data for '+str(len(pieces))+' files')
     logger.debug('headers: %r', headers)
@@ -528,23 +529,23 @@ def getpieces(f, encoding = None, progress = lambda x: None, separate_all = 1,
         if pkg.get('Size', '') and pkg.get('SHA1', '') and pkg.get('Filename', ''):
             # Check which torrent to add the info to
             all = 0
-            if (separate_all and pkg.get('Architecture', '') == 'all'):
+            if (separate_all and str(pkg.get('Architecture', '')) == 'all'):
                 all = 1
 
-            sha1 = binascii.a2b_hex(pkg['SHA1'])
+            sha1 = binascii.a2b_hex(str(pkg['SHA1']))
             if sub_pieces.has_key(sha1):
-                lengths[all][pkg['Filename']] = sub_pieces[sha1][1]
-                pieces[all][pkg['Filename']] = sub_pieces[sha1][0]
+                lengths[all][str(pkg['Filename'])] = sub_pieces[sha1][1]
+                pieces[all][str(pkg['Filename'])] = sub_pieces[sha1][0]
             else:
-                lengths[all][pkg['Filename']] = [long(pkg['Size'])]
-                pieces[all][pkg['Filename']] = [sha1]
+                lengths[all][str(pkg['Filename'])] = [long(str(pkg['Size']))]
+                pieces[all][str(pkg['Filename'])] = [sha1]
 
             path = []
-            temp = pkg['Filename']
+            temp = str(pkg['Filename'])
             while temp:
                 temp,d = split(temp)
                 path.insert(0,d)
-            fs[all][pkg['Filename']] = {'length': long(pkg['Size']), 'path': uniconvertl(path, encoding)}
+            fs[all][str(pkg['Filename'])] = {'length': long(str(pkg['Size'])), 'path': uniconvertl(path, encoding)}
             packages[all] += 1
             progress(packages[0] + packages[1])
             
