@@ -2,7 +2,7 @@
 # Modified by Cameron Dale
 # see LICENSE.txt for license information
 #
-# $Id: track.py 402 2008-07-26 00:43:37Z camrdale-guest $
+# $Id: track.py 412 2009-03-01 20:47:55Z camrdale-guest $
 
 """Tools to track a download swarm.
 
@@ -161,7 +161,7 @@ def statefiletemplate(x):
                     if type(info.get('requirecrypto')) not in (IntType,LongType):
                         raise ValueError
         elif cname == 'stats':
-            if (type(cinfo) != DictType): # The 'stats' key is a dictionary of SHA hashes (torrent ids)
+            if (type(cinfo) != DictType): # The 'stats' key is a dictionary of SHA hashes (and the special key 'global')
                 raise ValueError          # ... for keeping track of the total completions per torrent
             for y in cinfo.values():      # ... each torrent has an integer value
                 if type(y) != DictType:
@@ -169,10 +169,10 @@ def statefiletemplate(x):
                 completed = y.get('completed')
                 if type(completed) not in (IntType,LongType) or completed < 0:
                     raise ValueError
-                uploaded = y.get('uploaded', 0)
+                uploaded = y.get('uploaded', 0L)
                 if type(uploaded) not in (IntType,LongType) or uploaded < 0:
                     raise ValueError
-                downloaded = y.get('downloaded', 0)
+                downloaded = y.get('downloaded', 0L)
                 if type(downloaded) not in (IntType,LongType) or downloaded < 0:
                     raise ValueError
         elif cname == 'allowed':
@@ -210,6 +210,8 @@ def loadState(configdir):
     """
     
     tempstate = configdir.getState()
+    if not tempstate:
+        tempstate = {'version': 0}
     if tempstate:
         version = tempstate.get('version', 0)
         # Upgrade from the previous version
@@ -219,24 +221,46 @@ def loadState(configdir):
                 if 'completed' in tempstate:
                     for hash, num in tempstate['completed'].items():
                         tempstate['stats'][hash] = {'completed': num,
-                                                    'downloaded': 0,
-                                                    'uploaded': 0}
+                                                    'downloaded': 0L,
+                                                    'uploaded': 0L}
                     del tempstate['completed']
                 tempstate['version'] = 1
-                logger.info('successfully upgraded the previous state file')
+                version = 1
+                logger.info('successfully upgraded to state file v1')
             except:
                 logger.exception('old statefile corrupt; resetting')
                 tempstate = {'version': 1}
+                version = 1
+
+        if version < 2:
+            try:
+                tempstate['stats']['global'] = {'completed': 0,
+                                                'downloaded': 0L,
+                                                'uploaded': 0L}
+                tempstate['version'] = 2
+                version = 2
+                logger.info('successfully upgraded to state file v2')
+            except:
+                logger.exception('old statefile corrupt; resetting')
+                tempstate = {'version': 2}
+                version = 2
 
         # Test the state to make sure it's not been corrupted
         try:
             statefiletemplate(tempstate)
+            # Convert the stats to longs
+            for stat in tempstate['stats'].values():
+                stat['downloaded'] = long(stat['downloaded'])
+                stat['uploaded'] = long(stat['uploaded'])
             logger.info('successfully loaded the previous state file')
         except:
             logger.exception('statefile corrupt; resetting')
-            tempstate = {'version': 1}
-    else:
-        tempstate = {'version': 1}
+            tempstate = {'version': 2,
+                         'stats': {'global': {'completed': 0,
+                                              'downloaded': 0L,
+                                              'uploaded': 0L}
+                                  }
+                        }
     
     return tempstate
 
@@ -725,32 +749,33 @@ class Tracker:
                 else:
                     names = [ (None,hash) for hash in self.downloads.keys() ]
 
+            total_uploaded = 0L
+            total_downloaded = 0L
+
+            # Write the table headers
+            s.write('<table summary="files" border="1">\n\n' \
+                    '<tr><th>torrent name/<br><code>    info hash</code></th>\n')
+            if self.config['allowed_dir']:
+                s.write('<th align="right">size</th>\n')
+            s.write('<th align="right">peers</th>\n' \
+                    '<th align="right">downloaded</th>\n' \
+                    '<th align="right">uploaded</th>\n' \
+                    '<th align="right">saved</th></tr>\n\n')
+
             if not names:
-                s.write('<p>not tracking any files yet...</p>\n')
+                s.write('<tr><td>not tracking any files yet...</td></tr>\n')
             else:
                 names.sort()
                 number_files = 0
                 total_peers = 0
-                total_uploaded = 0L
-                total_downloaded = 0L
                 total_size = 0L
                 
-                # Write the table headers
-                s.write('<table summary="files" border="1">\n\n' \
-                        '<tr><th>torrent name/<br><code>    info hash</code></th>\n')
-                if self.config['allowed_dir']:
-                    s.write('<th align="right">size</th>\n')
-                s.write('<th align="right">peers</th>\n' \
-                        '<th align="right">downloaded</th>\n' \
-                        '<th align="right">uploaded</th>\n' \
-                        '<th align="right">saved</th></tr>\n\n')
-
                 # Display a table row for each torrent
                 for name,hash in names:
                     # Get the stats for this torrent
-                    uploaded = self.stats.get(hash, {}).get('uploaded', 0)
+                    uploaded = self.stats.get(hash, {}).get('uploaded', 0L)
                     total_uploaded += uploaded
-                    downloaded = self.stats.get(hash, {}).get('downloaded', 0)
+                    downloaded = self.stats.get(hash, {}).get('downloaded', 0L)
                     total_downloaded += downloaded
                     saved = 0.0
                     if downloaded > 0:
@@ -803,16 +828,32 @@ class Tracker:
                         % (total_peers, size_format(total_downloaded),
                            size_format(total_uploaded), saved))
 
-                # Write a rudimentary legend
-                s.write('</table>\n' \
-                    '<ul>\n' \
-                    '<li><em>torrent name:</em> possible names for the torrent</li>\n' \
-                    '<li><em>info hash:</em> torrent identifier</li>\n' \
-                    '<li><em>peers:</em> number of connected clients</li>\n' \
-                    '<li><em>downloaded:</em> amount of data downloaded by all peers, this data may have been downloaded from a mirror or another peer</li>\n' \
-                    '<li><em>uploaded:</em> amount of data uploaded by all peers, this is the amount of bandwidth that was saved from the mirrors</li>\n' \
-                    '<li><em>saved:</em> percentage of data that was NOT downloaded from a mirror</li>\n' \
-                    '</ul>\n')
+            # Another table footer with all-time statistics
+            total_downloaded += self.stats['global']['downloaded']
+            total_uploaded += self.stats['global']['uploaded']
+            saved = 0.0
+            if total_downloaded > 0:
+                saved = 100.0 * float(total_uploaded) / float(total_downloaded)
+            s.write('<tr><td align="right">All-time totals</td>\n')
+            if self.config['allowed_dir']:
+                s.write('<td></td>\n')
+            s.write('<td></td>\n' \
+                    '<td align="right">%s</td>\n' \
+                    '<td align="right">%s</td>\n' \
+                    '<td align="right">%0.1f %%</td></tr>\n\n'
+                    % (size_format(total_downloaded),
+                       size_format(total_uploaded), saved))
+
+            # Write a rudimentary legend
+            s.write('</table>\n' \
+                '<ul>\n' \
+                '<li><em>torrent name:</em> possible names for the torrent</li>\n' \
+                '<li><em>info hash:</em> torrent identifier</li>\n' \
+                '<li><em>peers:</em> number of connected clients</li>\n' \
+                '<li><em>downloaded:</em> amount of data downloaded by all peers, this data may have been downloaded from a mirror or another peer</li>\n' \
+                '<li><em>uploaded:</em> amount of data uploaded by all peers, this is the amount of bandwidth that was saved from the mirrors</li>\n' \
+                '<li><em>saved:</em> percentage of data that was NOT downloaded from a mirror</li>\n' \
+                '</ul>\n')
 
             s.write('</body>\n' \
                     '</html>\n')
@@ -1607,13 +1648,17 @@ class Tracker:
                 if t < self.prevtime:
                     self.delete_peer(x,myid)
         self.prevtime = clock()
-        if (self.keep_dead != 1):
-            for key, value in self.downloads.items():
-                if len(value) == 0 and (
-                        self.allowed is None or not self.allowed.has_key(key) ):
-                    del self.times[key]
-                    del self.downloads[key]
-                    del self.seedcount[key]
+        for key, value in self.downloads.items():
+            if len(value) == 0 and (
+                    self.allowed is None or not self.allowed.has_key(key) ) and (
+                    self.keep_dead != 1 or (self.stats.get(key, {}).get('uploaded', 0) <= 0 and
+                                            self.stats.get(key, {}).get('downloaded', 0) <= 0)):
+                self.stats['global']['downloaded'] += self.stats.get(key, {}).get('downloaded', 0L)
+                self.stats['global']['uploaded'] += self.stats.get(key, {}).get('uploaded', 0L)
+                self.stats['global']['completed'] += self.stats.get(key, {}).get('completed', 0L)
+                del self.times[key]
+                del self.downloads[key]
+                del self.seedcount[key]
         self.rawserver.add_task(self.expire_downloaders, self.timeout_downloaders_interval)
 
 
