@@ -1,7 +1,7 @@
 # Written by Cameron Dale
 # see LICENSE.txt for license information
 #
-# $Id: HTTPCache.py 382 2008-06-20 03:27:06Z camrdale-guest $
+# $Id: HTTPCache.py 413 2009-09-09 06:01:19Z camrdale $
 
 """Manage an HTTP download cache.
 
@@ -606,6 +606,9 @@ class HTTPCache:
         else:
             server_mtime = timegm(strptime(server_mtime_string, time_format+' %Z'))
 
+        if http_mtime_string is None:
+            return gmtime() - server_mtime
+        
         http_mtime = timegm(strptime(http_mtime_string, time_format+' %Z'))
 
         return http_mtime - server_mtime
