@@ -2,7 +2,7 @@
 # Modified by Cameron Dale
 # see LICENSE.txt for license information
 #
-# $Id: HTTPHandler.py 320 2007-11-07 00:06:50Z camrdale-guest $
+# $Id: HTTPHandler.py 376 2008-05-05 01:33:30Z camrdale-guest $
 
 """Handles incoming HTTP connections from other clients to this server.
 
@@ -20,7 +20,7 @@ from sys import stdout, exc_info
 import time
 from clock import clock
 from gzip import GzipFile
-import signal, logging
+import signal, os, logging
 
 logger = logging.getLogger('DebTorrent.HTTPHandler')
 
@@ -62,7 +62,7 @@ class HTTPRequest:
     @ivar encoding: the encoding to use when sending the response
     @type headers: C{dictionary}
     @ivar headers: the headers received with the request
-    @type answer: (C{int}, C{string}, C{dictionary}, C{string})
+    @type answer: (C{int}, C{string}, C{dictionary}, C{string} or C{file})
     @ivar answer: the HTTP status code, status message, headers, and package
         data, or None if the answer is not yet available
     
@@ -94,7 +94,7 @@ class HTTPRequest:
     def save_answer(self, r):
         """Save an answer, replacing the old one if it's better.
         
-        @type r: (C{int}, C{string}, C{dictionary}, C{string})
+        @type r: (C{int}, C{string}, C{dictionary}, C{string} or C{file})
         @param r: the HTTP status code, status message, headers, and package data
         
         """
@@ -368,7 +368,7 @@ class HTTPConnection:
     def answer(self, r, httpreq):
         """Add a response to the queued responses and check if any are ready to send.
         
-        @type r: (C{int}, C{string}, C{dictionary}, C{string})
+        @type r: (C{int}, C{string}, C{dictionary}, C{string} or C{file})
         @param r: the HTTP status code, status message, headers, and package data
         @type httpreq: L{HTTPRequest}
         @param httpreq: the request the answer is for
@@ -427,7 +427,7 @@ class HTTPConnection:
         @param responsestring: the response string to send
         @type headers: C{dictionary}
         @param headers: the headers to send with the response
-        @type data: C{string}
+        @type data: C{string} or C{file}
         @param data: the data to send with the response
         @type header: C{string}
         @param header: the first header line received from the request
@@ -443,7 +443,7 @@ class HTTPConnection:
         """
 
         # Encode the response data
-        if encoding == 'gzip':
+        if encoding == 'gzip' and isinstance(data, str):
             compressed = StringIO()
             gz = GzipFile(fileobj = compressed, mode = 'wb', compresslevel = 9)
             gz.write(data)
@@ -456,18 +456,37 @@ class HTTPConnection:
                 data = cdata
                 headers['Content-Encoding'] = 'gzip'
 
+        # Determine the length of the data to be written
+        if isinstance(data, str):
+            size = len(data)
+        else:
+            size = os.fstat(data.fileno())[6] - data.tell()
+
+        # Check for a local header and other qualifications
+        if (req_headers.get('if-local-nocache', '').lower() == 'true' and
+            self.protocol == "DEBTORRENT" and type(data) == file and
+            responsecode == 200 and self.connection.get_ip().startswith('127.')):
+            # Send the No Content response with the file name as header
+            responsecode = 204
+            responsestring = "No Content"
+            headers['NoCache-File'] = data.name
+            data.close()
+            data = None
+        
         # i'm abusing the identd field here, but this should be ok
         if encoding == 'identity':
             ident = '-'
         else:
             ident = self.encoding
-        self.handler.write_log( self.connection.get_ip(), ident, '-',
-                                header, responsecode, len(data),
-                                req_headers.get('referer', '-'),
-                                req_headers.get('user-agent', '-') )
-
-        logger.info('sending response: '+self.protocol+' '+str(responsecode)+' '+responsestring+
-                    ' ('+str(len(data))+' bytes)' + repr(headers))
+            
+        if responsecode >= 200:
+            self.handler.write_log( self.connection.get_ip(), ident, '-',
+                                    header, responsecode, size,
+                                    req_headers.get('referer', '-'),
+                                    req_headers.get('user-agent', '-') )
+    
+            logger.info('sending response: '+self.protocol+' '+str(responsecode)+' '+responsestring+
+                        ' ('+str(size)+' bytes)' + repr(headers))
         
         r = StringIO()
         
@@ -481,17 +500,17 @@ class HTTPConnection:
             
         # Write the individual headers
         if self.version >= (1, 0) or self.protocol != 'HTTP':
-            headers['Content-Length'] = len(data)
+            headers['Content-Length'] = size
             for key, value in headers.items():
                 r.write(key + ': ' + str(value) + '\r\n')
             r.write('\r\n')
             
-        # Don't write the body if only the headers are requested
-        if command != 'HEAD' or not data:
-            r.write(data)
-            
         self.connection.write(r.getvalue())
     
+        # Don't write the body if only the headers are requested
+        if command != 'HEAD' and data:
+            self.connection.write(data)
+            
     def close(self):
         """Close the connection and drop all pending requests/answers."""
         logger.debug('HTTP connection closed')
