@@ -2,7 +2,7 @@
 # Modified by Cameron Dale
 # see LICENSE.txt for license information
 #
-# $Id: ServerPortHandler.py 270 2007-08-19 07:33:13Z camrdale-guest $
+# $Id: ServerPortHandler.py 389 2008-06-22 19:23:06Z camrdale-guest $
 
 """Wrappers to handle multiple torrent downloads.
 
@@ -33,6 +33,8 @@ class SingleRawServer:
     
     @type info_hash: C{string}
     @ivar info_hash: the torrent infohash this Server is responsible for
+    @type identifier: C{string}
+    @ivar identifier: the identifier of the torrent
     @type doneflag: C{threading.Event}
     @ivar doneflag: flag to indicate this torrent is being shutdown
     @type protocol: C{string}
@@ -52,11 +54,13 @@ class SingleRawServer:
     
     """
     
-    def __init__(self, info_hash, multihandler, doneflag, protocol):
+    def __init__(self, info_hash, identifier, multihandler, doneflag, protocol):
         """Initialize the instance.
         
         @type info_hash: C{string}
         @param info_hash: the torrent infohash this Server is responsible for
+        @type identifier: C{string}
+        @param identifier: the identifier of the torrent
         @type multihandler: L{MultiHandler}
         @param multihandler: the collection of all individual simplified servers
         @type doneflag: C{threading.Event}
@@ -67,6 +71,7 @@ class SingleRawServer:
         """
         
         self.info_hash = info_hash
+        self.identifier = identifier
         self.doneflag = doneflag
         self.protocol = protocol
         self.multihandler = multihandler
@@ -79,7 +84,7 @@ class SingleRawServer:
     def shutdown(self):
         """Tell the collection to shutdown this torrent."""
         if not self.finished:
-            self.multihandler.shutdown_torrent(self.info_hash)
+            self.multihandler.shutdown_torrent(self.identifier)
 
     def _shutdown(self):
         """Shutdown this torrent."""
@@ -554,11 +559,13 @@ class MultiHandler:
         self.connections = {}
         self.taskqueues = {}
 
-    def newRawServer(self, info_hash, doneflag, protocol=protocol_name):
+    def newRawServer(self, info_hash, identifier, doneflag, protocol=protocol_name):
         """Create a new Server for the torrent.
         
         @type info_hash: C{string}
         @param info_hash: the torrent's infohash
+        @type identifier: C{string}
+        @param identifier: the identifier of the torrent
         @type doneflag: C{threading.Event}
         @param doneflag: the flag to indicate stopping to the new Server
         @type protocol: C{string}
@@ -569,20 +576,20 @@ class MultiHandler:
         
         """
         
-        new = SingleRawServer(info_hash, self, doneflag, protocol)
-        self.singlerawservers[info_hash] = new
+        new = SingleRawServer(info_hash, identifier, self, doneflag, protocol)
+        self.singlerawservers[identifier] = new
         return new
 
-    def shutdown_torrent(self, info_hash):
+    def shutdown_torrent(self, identifier):
         """Shutdown a single torrent's Server.
         
-        @type info_hash: C{string}
-        @param info_hash: the torrent's infohash
+        @type identifier: C{string}
+        @param identifier: the identifier of the torrent
 
         """
         
-        self.singlerawservers[info_hash]._shutdown()
-        del self.singlerawservers[info_hash]
+        self.singlerawservers[identifier]._shutdown()
+        del self.singlerawservers[identifier]
 
     def listen_forever(self):
         """Call the master server's listen loop.
