@@ -2,7 +2,7 @@
 # Modified by Cameron Dale
 # see LICENSE.txt for license information
 #
-# $Id: SocketHandler.py 266 2007-08-18 02:06:35Z camrdale-guest $
+# $Id: SocketHandler.py 364 2008-01-27 22:38:02Z camrdale-guest $
 
 """Handle all sockets.
 
@@ -65,9 +65,9 @@ class SingleSocket:
         @param sock: the socket to manage
         @type handler: unknown
         @param handler: the handler to use for all communications on the socket
-        @type ip: C{string}
-        @param ip: the IP address to use if one can't be obtained from the socket
-            (optional, defaults to 'unknown')
+        @type dns: (C{string}, C{int})
+        @param dns: the IP address and port to use if one can't be obtained
+            from the socket (optional, defaults to 'unknown')
         
         """
         
@@ -167,8 +167,8 @@ class SingleSocket:
         Adds the data to the buffer of data waiting to be written, then tries 
         to write the waiting data out.
         
-        @type s: C{string}
-        @param s: the data to write
+        @type s: C{string} or C{file}
+        @param s: the data to write, or an already opened file to write out
         
         """
         
@@ -190,6 +190,20 @@ class SingleSocket:
             dead = False
             try:
                 while self.buffer:
+                    # Read data from the file and put in on the buffer
+                    while self.buffer and type(self.buffer[0]) == file:
+                        data = self.buffer[0].read(4096)
+                        if len(data) > 0:
+                            self.buffer.insert(0, data)
+                        else:
+                            # End of file has been reached
+                            self.buffer[0].close()
+                            del self.buffer[0]
+
+                    # Make sure there's still data to send
+                    if not self.buffer:
+                        break
+                            
                     buf = self.buffer[0]
                     amount = self.socket.send(buf)
                     if amount == 0:
