@@ -2,7 +2,7 @@
 # Modified by Cameron Dale
 # see LICENSE.txt for license information
 #
-# $Id: bitfield.py 333 2008-01-19 21:32:11Z camrdale-guest $
+# $Id: bitfield.py 416 2010-03-20 18:29:37Z camrdale $
 
 """Manage creation and modification of bitfields.
 
@@ -55,7 +55,8 @@ class Bitfield:
     
     """
     
-    def __init__(self, length = None, bitstring = None, copyfrom = None):
+    def __init__(self, length = None, bitstring = None, copyfrom = None,
+                 defaultValue = False):
         """
         
         @type length: C{int}
@@ -63,10 +64,13 @@ class Bitfield:
             (optional, if missing use length of copyfrom)
         @type bitstring: C{string}
         @param bitstring: the bitfield string to initialize the bitfield from
-            (optional, default is to initialize all to false)
+            (optional, default is to initialize all to L{defaultValue})
         @type copyfrom: L{Bitfield}
         @param copyfrom: another bitfield to make a copy of
             (optional, default is to create a new empty one)
+        @type defaultValue: C{boolean}
+        @param defaultValue: the default value to initialize the bitfield with
+            (optional, default is to initialize all to false)
         
         """
         
@@ -98,8 +102,11 @@ class Bitfield:
             self.array = r
             self.numfalse = negsum(r)
         else:
-            self.array = [False] * length
-            self.numfalse = length
+            self.array = [defaultValue] * length
+            if defaultValue:
+                self.numfalse = 0
+            else:
+                self.numfalse = length
 
     def __setitem__(self, index, val):
         """Set one of the bitfield entries.
