@@ -2,7 +2,7 @@
 # Modified by Cameron Dale
 # see LICENSE.txt for license information
 #
-# $Id: download_bt1.py 402 2008-07-26 00:43:37Z camrdale-guest $
+# $Id: download_bt1.py 416 2010-03-20 18:29:37Z camrdale $
 
 """Manage a single download.
 
@@ -1198,7 +1198,7 @@ class BT1Download:
 
         self.httpdownloader = HTTPDownloader(self.storagewrapper, self.picker,
             self.rawserver, self.finflag, self.errorfunc, self.downloader,
-            self.config['max_rate_period'], self.infohash, self._received_http_data,
+            self.config['max_rate_period'], self.len_pieces, self._received_http_data,
             self.connecter.got_piece, self.getFilename)
         if self.response.has_key('deb_mirrors') and not self.finflag.isSet():
             if not self.config['disable_http_downloader']:
