@@ -2,7 +2,7 @@
 # Modified by Cameron Dale
 # see LICENSE.txt for license information
 #
-# $Id: launchmanycore.py 401 2008-07-26 00:36:45Z camrdale-guest $
+# $Id: launchmanycore.py 415 2009-09-17 03:37:07Z camrdale $
 
 """Manage the downloading of multiple torrents in one process.
 
@@ -35,6 +35,7 @@ from cStringIO import StringIO
 import logging
 from DebTorrent.BT1.AptListener import AptListener
 from DebTorrent.HTTPHandler import HTTPHandler
+from binascii import b2a_hex, a2b_hex
 
 logger = logging.getLogger('DebTorrent.launchmanycore')
 
@@ -497,6 +498,38 @@ class LaunchMany:
 
         return data
 
+    def get_piecemap(self, id):
+        """Return the piecemap (which peers have what) for a given torrent.
+
+        @type id: C{string}
+        @param id: Info hash for the torrent, as provided by gather_stats.
+
+        Internal: The "id" argument should match the identifier that
+        gather_stats provides in data[][1].  As of revision 373 it provides the
+        long-lived torrent ID, not the hash.
+
+        @rtype: C{list}
+        @return: List of piecemaps.  The first will be for the local peer, then the other peers in undefined order.
+
+        """
+
+        hash = None
+        for search_hash in self.torrent_list:
+            if self.torrent_cache[search_hash]['metainfo'].get('identifier', search_hash) == id:
+                hash = search_hash
+
+        if hash == None:
+            logger.warning('Could not find hash for id %s\n' % b2a_hex(id))
+            return None
+
+        piecelist_list = []
+        piecelist_list.append(self.downloads[hash].d.storagewrapper.have)
+        if (self.downloads[hash].d.downloader != None):
+            for peer in self.downloads[hash].d.downloader.downloads:
+                piecelist_list.append(peer.have)
+
+        return piecelist_list
+
     def remove(self, hash):
         """Stop and remove a running torrent.
         
