@@ -2,7 +2,7 @@
 #
 # see LICENSE.txt for license information
 #
-# $Id: test.py 318 2007-11-06 21:21:25Z camrdale-guest $
+# $Id: test.py 381 2008-06-19 22:40:22Z camrdale-guest $
 
 """Automated tests of the DebTorrent functionality.
 
@@ -50,16 +50,14 @@ tests = {'1': ('Start a single tracker and downloader, test updating and downloa
              {1: []},
              {1: (1, [], {})},
              [(1, ['update']), 
-              (1, ['install', 'aboot-base']),
-              (1, ['install', 'aap-doc']),
-              (1, ['install', 'ada-reference-manual']),
               (1, ['install', 'aspectj-doc']),
               (1, ['install', 'fop-doc']),
-              (1, ['install', 'jswat-doc']),
               (1, ['install', 'asis-doc']),
               (1, ['install', 'bison-doc']),
               (1, ['install', 'crash-whitepaper']),
-              (1, ['install', 'doc-iana']),
+              (1, ['install', 'aboot-base']),
+              (1, ['install', 'aap-doc']),
+              (1, ['install', 'ada-reference-manual']),
               ]),
 
          '2': ('Start a single tracker and 2 downloaders, disable HTTP downloading' +
@@ -68,21 +66,19 @@ tests = {'1': ('Start a single tracker and downloader, test updating and downloa
                {1: (1, [], {}),
                 2: (1, ['--disable_http_downloader', '1'], {})},
                [(1, ['update']),
+                (1, ['install', 'fop-doc']),
+                (1, ['install', 'bison-doc']),
+                (1, ['install', 'crash-whitepaper']),
                 (1, ['install', 'aboot-base']),
                 (1, ['install', 'aap-doc']),
                 (1, ['install', 'ada-reference-manual']),
-                (1, ['install', 'fop-doc']),
-                (1, ['install', 'jswat-doc']),
-                (1, ['install', 'bison-doc']),
-                (1, ['install', 'crash-whitepaper']),
                 (2, ['update']),
+                (2, ['install', 'fop-doc']),
+                (2, ['install', 'bison-doc']),
+                (2, ['install', 'crash-whitepaper']),
                 (2, ['install', 'aboot-base']),
                 (2, ['install', 'aap-doc']),
                 (2, ['install', 'ada-reference-manual']),
-                (2, ['install', 'fop-doc']),
-                (2, ['install', 'jswat-doc']),
-                (2, ['install', 'bison-doc']),
-                (2, ['install', 'crash-whitepaper']),
                 ]),
                 
          '3': ('Start a single tracker and 6 downloaders, to test downloading' +
@@ -95,35 +91,29 @@ tests = {'1': ('Start a single tracker and downloader, test updating and downloa
                 5: (1, [], {}),
                 6: (1, [], {})},
                [(1, ['update']),
+                (1, ['install', 'fop-doc']),
                 (1, ['install', 'aboot-base']),
                 (1, ['install', 'ada-reference-manual']),
-                (1, ['install', 'fop-doc']),
-                (1, ['install', 'doc-iana']),
                 (2, ['update']),
+                (2, ['install', 'fop-doc']),
                 (2, ['install', 'aboot-base']),
                 (2, ['install', 'ada-reference-manual']),
-                (2, ['install', 'fop-doc']),
-                (2, ['install', 'doc-iana']),
                 (3, ['update']),
+                (3, ['install', 'fop-doc']),
                 (3, ['install', 'aboot-base']),
                 (3, ['install', 'ada-reference-manual']),
-                (3, ['install', 'fop-doc']),
-                (3, ['install', 'doc-iana']),
                 (4, ['update']),
+                (4, ['install', 'fop-doc']),
                 (4, ['install', 'aboot-base']),
                 (4, ['install', 'ada-reference-manual']),
-                (4, ['install', 'fop-doc']),
-                (4, ['install', 'doc-iana']),
                 (5, ['update']),
+                (5, ['install', 'fop-doc']),
                 (5, ['install', 'aboot-base']),
                 (5, ['install', 'ada-reference-manual']),
-                (5, ['install', 'fop-doc']),
-                (5, ['install', 'doc-iana']),
                 (6, ['update']),
+                (6, ['install', 'fop-doc']),
                 (6, ['install', 'aboot-base']),
                 (6, ['install', 'ada-reference-manual']),
-                (6, ['install', 'fop-doc']),
-                (6, ['install', 'doc-iana']),
                 ]),
 
          '4': ('Start a single tracker and 1 downloader, requesting the same' +
@@ -131,31 +121,28 @@ tests = {'1': ('Start a single tracker and downloader, test updating and downloa
                {1: []},
                {1: (1, [], {})},
                [(1, ['update']),
+                (1, ['install', 'fop-doc']),
                 (1, ['install', 'aboot-base']),
                 (1, ['install', 'ada-reference-manual']),
-                (1, ['install', 'fop-doc']),
-                (1, ['install', 'doc-iana']),
                 (1, ['update']),
+                (1, ['install', 'fop-doc']),
                 (1, ['install', 'aboot-base']),
                 (1, ['install', 'ada-reference-manual']),
-                (1, ['install', 'fop-doc']),
-                (1, ['install', 'doc-iana']),
                 (1, ['update']),
+                (1, ['install', 'fop-doc']),
                 (1, ['install', 'aboot-base']),
                 (1, ['install', 'ada-reference-manual']),
-                (1, ['install', 'fop-doc']),
-                (1, ['install', 'doc-iana']),
                 ]),
                 
          '5': ('Start a single tracker and 6 downloaders, update all to test' +
                ' that they can all see each other.',
                {1: []},
                {1: (1, [], {'suites': 'contrib non-free'}),
-                2: (1, [], {'suites': 'contrib non-free'}),
-                3: (1, [], {'suites': 'contrib non-free'}),
-                4: (1, [], {'suites': 'contrib non-free'}),
-                5: (1, [], {'suites': 'contrib non-free'}),
-                6: (1, [], {'suites': 'contrib non-free'})},
+                2: (1, [], {'suites': 'contrib non-free', 'mirror': 'debian.mirror.iweb.ca/debian'}),
+                3: (1, [], {'suites': 'contrib non-free', 'mirror': 'debian.sth.sze.hu'}),
+                4: (1, [], {'suites': 'contrib non-free', 'mirror': 'ftp.monash.edu.au/pub/linux/debian'}),
+                5: (1, [], {'suites': 'contrib non-free', 'mirror': 'mmc.igeofcu.unam.mx/debian'}),
+                6: (1, [], {'suites': 'contrib non-free', 'mirror': 'debian.revolsys.fr/debian'})},
                [(1, ['update']),
                 (2, ['update']),
                 (3, ['update']),
@@ -178,11 +165,11 @@ tests = {'1': ('Start a single tracker and downloader, test updating and downloa
              {1: (1, [], {})},
              [(1, ['update']), 
               (1, ['install', 'aboot-base', 'aap-doc', 'ada-reference-manual',
-                   'aspectj-doc', 'fop-doc', 'jswat-doc', 'asis-doc',
-                   'bison-doc', 'crash-whitepaper', 'doc-iana',
+                   'aspectj-doc', 'fop-doc', 'asis-doc',
+                   'bison-doc', 'crash-whitepaper',
                    'bash-doc', 'apt-howto-common', 'autotools-dev',
-                   'aptitude-doc-en', 'armagetron-common', 'asr-manpages',
-                   'atomix-data', 'alcovebook-sgml-doc', 'alamin-doc',
+                   'aptitude-doc-en', 'asr-manpages',
+                   'atomix-data', 'alcovebook-sgml-doc',
                    'aegis-doc', 'afbackup-common', 'airstrike-common',
                    ]),
               ]),
@@ -197,60 +184,79 @@ tests = {'1': ('Start a single tracker and downloader, test updating and downloa
               6: (1, [], {})},
              [(1, ['update']), 
               (1, ['install', 'aboot-base', 'aap-doc', 'ada-reference-manual',
-                   'aspectj-doc', 'fop-doc', 'jswat-doc', 'asis-doc',
-                   'bison-doc', 'crash-whitepaper', 'doc-iana',
+                   'aspectj-doc', 'fop-doc', 'asis-doc',
+                   'bison-doc', 'crash-whitepaper',
                    'bash-doc', 'apt-howto-common', 'autotools-dev',
-                   'aptitude-doc-en', 'armagetron-common', 'asr-manpages',
-                   'atomix-data', 'alcovebook-sgml-doc', 'alamin-doc',
+                   'aptitude-doc-en', 'asr-manpages',
+                   'atomix-data', 'alcovebook-sgml-doc',
                    'aegis-doc', 'afbackup-common', 'airstrike-common',
                    ]),
               (2, ['update']), 
               (2, ['install', 'aboot-base', 'aap-doc', 'ada-reference-manual',
-                   'aspectj-doc', 'fop-doc', 'jswat-doc', 'asis-doc',
-                   'bison-doc', 'crash-whitepaper', 'doc-iana',
+                   'aspectj-doc', 'fop-doc', 'asis-doc',
+                   'bison-doc', 'crash-whitepaper',
                    'bash-doc', 'apt-howto-common', 'autotools-dev',
-                   'aptitude-doc-en', 'armagetron-common', 'asr-manpages',
-                   'atomix-data', 'alcovebook-sgml-doc', 'alamin-doc',
+                   'aptitude-doc-en', 'asr-manpages',
+                   'atomix-data', 'alcovebook-sgml-doc',
                    'aegis-doc', 'afbackup-common', 'airstrike-common',
                    ]),
               (3, ['update']), 
               (3, ['install', 'aboot-base', 'aap-doc', 'ada-reference-manual',
-                   'aspectj-doc', 'fop-doc', 'jswat-doc', 'asis-doc',
-                   'bison-doc', 'crash-whitepaper', 'doc-iana',
+                   'aspectj-doc', 'fop-doc', 'asis-doc',
+                   'bison-doc', 'crash-whitepaper',
                    'bash-doc', 'apt-howto-common', 'autotools-dev',
-                   'aptitude-doc-en', 'armagetron-common', 'asr-manpages',
-                   'atomix-data', 'alcovebook-sgml-doc', 'alamin-doc',
+                   'aptitude-doc-en', 'asr-manpages',
+                   'atomix-data', 'alcovebook-sgml-doc',
                    'aegis-doc', 'afbackup-common', 'airstrike-common',
                    ]),
               (4, ['update']), 
               (4, ['install', 'aboot-base', 'aap-doc', 'ada-reference-manual',
-                   'aspectj-doc', 'fop-doc', 'jswat-doc', 'asis-doc',
-                   'bison-doc', 'crash-whitepaper', 'doc-iana',
+                   'aspectj-doc', 'fop-doc', 'asis-doc',
+                   'bison-doc', 'crash-whitepaper',
                    'bash-doc', 'apt-howto-common', 'autotools-dev',
-                   'aptitude-doc-en', 'armagetron-common', 'asr-manpages',
-                   'atomix-data', 'alcovebook-sgml-doc', 'alamin-doc',
+                   'aptitude-doc-en', 'asr-manpages',
+                   'atomix-data', 'alcovebook-sgml-doc',
                    'aegis-doc', 'afbackup-common', 'airstrike-common',
                    ]),
               (5, ['update']), 
               (5, ['install', 'aboot-base', 'aap-doc', 'ada-reference-manual',
-                   'aspectj-doc', 'fop-doc', 'jswat-doc', 'asis-doc',
-                   'bison-doc', 'crash-whitepaper', 'doc-iana',
+                   'aspectj-doc', 'fop-doc', 'asis-doc',
+                   'bison-doc', 'crash-whitepaper',
                    'bash-doc', 'apt-howto-common', 'autotools-dev',
-                   'aptitude-doc-en', 'armagetron-common', 'asr-manpages',
-                   'atomix-data', 'alcovebook-sgml-doc', 'alamin-doc',
+                   'aptitude-doc-en', 'asr-manpages',
+                   'atomix-data', 'alcovebook-sgml-doc',
                    'aegis-doc', 'afbackup-common', 'airstrike-common',
                    ]),
               (6, ['update']), 
               (6, ['install', 'aboot-base', 'aap-doc', 'ada-reference-manual',
-                   'aspectj-doc', 'fop-doc', 'jswat-doc', 'asis-doc',
-                   'bison-doc', 'crash-whitepaper', 'doc-iana',
+                   'aspectj-doc', 'fop-doc', 'asis-doc',
+                   'bison-doc', 'crash-whitepaper',
                    'bash-doc', 'apt-howto-common', 'autotools-dev',
-                   'aptitude-doc-en', 'armagetron-common', 'asr-manpages',
-                   'atomix-data', 'alcovebook-sgml-doc', 'alamin-doc',
+                   'aptitude-doc-en', 'asr-manpages',
+                   'atomix-data', 'alcovebook-sgml-doc',
                    'aegis-doc', 'afbackup-common', 'airstrike-common',
                    ]),
               ]),
 
+        '9': ('Run this test multiple times to test restarting the downloader.',
+             {1: []},
+             {1: (1, [], {'clean': False})},
+             [(1, ['update']), 
+              (1, ['install', 'aspectj-doc']),
+              (1, ['install', 'fop-doc']),
+              (1, ['install', 'asis-doc']),
+              (1, ['install', 'bison-doc']),
+              (1, ['install', 'crash-whitepaper']),
+              (1, ['install', 'libhypre1.6.0c2']),
+              (1, ['install', 'rar']),
+              (1, ['install', 'aboot-base']),
+              (1, ['install', 'aap-doc']),
+              (1, ['install', 'ada-reference-manual']),
+              (1, ['install', '2vcard']),
+              (1, ['install', 'avahi-dbg']),
+              (1, ['install', 'chasen-cannadic']),
+              ]),
+
          }
 
 assert 'all' not in tests
@@ -296,6 +302,19 @@ apt_conf_template = """
   };
 };
 
+// Options for the downloading routines
+Acquire
+{
+  // DEBTORRENT method configuration
+  debtorrent
+  {
+    // The timeout delay before the method will close the connection
+    Timeout "300";
+    // Don't keep copies of the downloaded files in apt's cache
+    NoCache "false";
+  };
+};
+
 /* Options you can set to see some debugging text They correspond to names
    of classes in the source code */
 Debug
@@ -330,11 +349,12 @@ def rmrf(top):
     
     """
     
-    for root, dirs, files in os.walk(top, topdown=False):
-        for name in files:
-            os.remove(os.path.join(root, name))
-        for name in dirs:
-            os.rmdir(os.path.join(root, name))
+    if exists(top):
+        for root, dirs, files in os.walk(top, topdown=False):
+            for name in files:
+                os.remove(os.path.join(root, name))
+            for name in dirs:
+                os.rmdir(os.path.join(root, name))
 
 def join(dir):
     """Join together a list of directories into a path string.
@@ -359,7 +379,8 @@ def makedirs(dir):
     
     """
     
-    os.makedirs(join(dir))
+    if not exists(join(dir)):
+        os.makedirs(join(dir))
 
 def touch(path):
     """Create an empty file.
@@ -368,9 +389,10 @@ def touch(path):
     @param path: the path to create
     
     """
-    
-    f = open(join(path), 'w')
-    f.close()
+
+    if not exists(join(path)):
+        f = open(join(path), 'w')
+        f.close()
 
 def start(func, args, work_dir = None):
     """Fork and start a background process running.
@@ -536,26 +558,29 @@ def start_downloader(num_down, options = [], mirror = 'ftp.us.debian.org/debian'
         except:
             pass
     
-        # Create the directory structure needed by apt
-        makedirs([downloader_dir, 'etc', 'apt', 'apt.conf.d'])
-        makedirs([downloader_dir, 'var', 'lib', 'apt', 'lists', 'partial'])
-        makedirs([downloader_dir, 'var', 'lib', 'dpkg'])
-        makedirs([downloader_dir, 'var', 'cache', 'apt', 'archives', 'partial'])
-        touch([downloader_dir, 'var', 'lib', 'apt', 'lists', 'lock'])
-        touch([downloader_dir, 'var', 'lib', 'dpkg', 'lock'])
-        touch([downloader_dir, 'var', 'lib', 'dpkg', 'status'])
-        touch([downloader_dir, 'var', 'cache', 'apt', 'archives', 'lock'])
+    # Create the directory structure needed by apt
+    makedirs([downloader_dir, 'etc', 'apt', 'apt.conf.d'])
+    makedirs([downloader_dir, 'var', 'lib', 'apt', 'lists', 'partial'])
+    makedirs([downloader_dir, 'var', 'lib', 'dpkg'])
+    rmrf(join([downloader_dir, 'var', 'cache', 'apt', 'archives']))
+    makedirs([downloader_dir, 'var', 'cache', 'apt', 'archives', 'partial'])
+    touch([downloader_dir, 'var', 'lib', 'apt', 'lists', 'lock'])
+    touch([downloader_dir, 'var', 'lib', 'dpkg', 'lock'])
+    touch([downloader_dir, 'var', 'lib', 'dpkg', 'status'])
+    touch([downloader_dir, 'var', 'cache', 'apt', 'archives', 'lock'])
 
+    if not exists(join([downloader_dir, 'etc', 'apt', 'sources.list'])):
         # Create apt's config files
         f = open(join([downloader_dir, 'etc', 'apt', 'sources.list']), 'w')
         if exists('/usr/lib/apt/methods/newdebtorrent'):
-            f.write('deb newdebtorrent://localhost:' + str(num_down) + '988/' + mirror + '/ stable ' + suites + '\n')
+            f.write('deb newdebtorrent://localhost:' + str(num_down) + '988/' + mirror + '/ unstable ' + suites + '\n')
         elif exists('/usr/lib/apt/methods/debtorrent'):
-            f.write('deb debtorrent://localhost:' + str(num_down) + '988/' + mirror + '/ stable ' + suites + '\n')
+            f.write('deb debtorrent://localhost:' + str(num_down) + '988/' + mirror + '/ unstable ' + suites + '\n')
         else:
-            f.write('deb http://localhost:' + str(num_down) + '988/' + mirror + '/ stable ' + suites + '\n')
+            f.write('deb http://localhost:' + str(num_down) + '988/' + mirror + '/ unstable ' + suites + '\n')
         f.close()
 
+    if not exists(join([downloader_dir, 'etc', 'apt', 'apt.conf'])):
         f = open(join([downloader_dir, 'etc', 'apt', 'apt.conf']), 'w')
         f.write('Dir "' + downloader_dir + '"')
         f.write(apt_conf_template)
@@ -641,7 +666,7 @@ def run_test(trackers, downloaders, apt_get_queue):
         sleep(5)
         
         for k, v in downloaders.items():
-            opts = v[1] + ['--default_tracker', tracker_address(v[0])]
+            opts = v[1] + ['--force_tracker', tracker_address(v[0])]
             running_downloaders[k] = start_downloader(k, opts, **v[2])
     
         sleep(10)
@@ -660,7 +685,9 @@ def run_test(trackers, downloaders, apt_get_queue):
             else:
                 print '********************** apt-get finished with status ' + str(r_value) + ' in ' +  str(elapsed) + ' sec. **************************'
         
-            sleep(15)
+            sleep(5)
+            if 'update' in cmd:
+                sleep(25)
             
     except:
         print '************************** Exception occurred **************************'
@@ -743,4 +770,4 @@ if __name__ == '__main__':
     else:
         print 'Unknown test to run:', sys.argv[1], '\n'
         print get_usage()
-        
\ No newline at end of file
+        
