@@ -10,7 +10,7 @@ from math import ceil
 from os import remove, rename, system, symlink
 from os.path import exists
 from time import strftime, gmtime
-from debian_bundle import deb822
+from debian import deb822
 from tempfile import mkstemp
 
 # The piece size to use (must match the '-extrapieces' file's piece size)
